{% extends "layout.jinja2" %}

{% block content %}
<div class="content">
  <h1><span class="font-semi-bold">c2cgeoform</span> <span class="smaller">demo project</span></h1>
  <p class="lead">Welcome to <span class="font-normal">c2cgeoform_demo</span>, a&nbsp;Pyramid application generated&nbsp;by<br><span class="font-normal">c2cgeoform</span>.</p>
</div>
{% endblock content %}
